@@ -8,15 +8,19 @@ module Lookup
 
 # @api private
 class ScopeLookupCollectingInvocation < Invocation
-  attr_reader :scope_interpolations
-
   def initialize(scope)
     super(scope)
     @scope_interpolations = []
   end
 
-  def remember_scope_lookup(*lookup_result)
-    @scope_interpolations << lookup_result
+  def remember_scope_lookup(key, root_key, segments, value)
+    @scope_interpolations << [key, root_key, segments, value] unless !value.nil? && key.start_with?('::')
+  end
+
+  def scope_interpolations
+    # Save extra checks by keeping the array unique with respect to the key (first entry)
+    @scope_interpolations.uniq! { |si| si[0] }
+    @scope_interpolations
   end
 end
 
@@ -48,6 +52,7 @@ class HieraConfig
   KEY_V3_BACKEND = V3BackendFunctionProvider::TAG
   KEY_V4_DATA_HASH = V4DataHashFunctionProvider::TAG
   KEY_BACKEND = 'backend'.freeze
+  KEY_EXTENSION = 'extension'.freeze
 
   FUNCTION_KEYS = [KEY_DATA_HASH, KEY_LOOKUP_KEY, KEY_DATA_DIG, KEY_V3_BACKEND]
   ALL_FUNCTION_KEYS = FUNCTION_KEYS + [KEY_V4_DATA_HASH]
@@ -99,22 +104,28 @@ class HieraConfig
   # Creates a new HieraConfig from the given _config_root_. This is where the 'hiera.yaml' is expected to be found
   # and is also the base location used when resolving relative paths.
   #
+  # @param lookup_invocation [Invocation] Invocation data containing scope, overrides, and defaults
   # @param config_path [Pathname] Absolute path to the configuration file
   # @return [LookupConfiguration] the configuration
-  def self.create(config_path)
+  def self.create(lookup_invocation, config_path)
     if config_path.is_a?(Hash)
       config_path = nil
       loaded_config = config_path
     else
       config_root = config_path.parent
       if config_path.exist?
-        loaded_config = YAML.load_file(config_path)
+        env_context = EnvironmentContext.adapt(lookup_invocation.scope.compiler.environment)
+        loaded_config = env_context.cached_file_data(config_path) do |content|
+          parsed = YAML.load(content, config_path)
 
-        # For backward compatibility, we must treat an empty file, or a yaml that doesn't
-        # produce a Hash as Hiera version 3 default.
-        unless loaded_config.is_a?(Hash)
-          Puppet.warning("#{config_path}: File exists but does not contain a valid YAML hash. Falling back to Hiera version 3 default config")
-          loaded_config = HieraConfigV3::DEFAULT_CONFIG_HASH
+          # For backward compatibility, we must treat an empty file, or a yaml that doesn't
+          # produce a Hash as Hiera version 3 default.
+          if parsed.is_a?(Hash)
+            parsed
+          else
+            Puppet.warning("#{config_path}: File exists but does not contain a valid YAML hash. Falling back to Hiera version 3 default config")
+            HieraConfigV3::DEFAULT_CONFIG_HASH
+          end
         end
       else
         config_path = nil
@@ -169,15 +180,21 @@ class HieraConfig
   end
 
   def scope_interpolations_stable?(lookup_invocation)
-    scope = lookup_invocation.scope
-    @scope_interpolations.all? do |key, root_key, segments, old_value|
-      value = scope[root_key]
-      unless value.nil? || segments.empty?
-        found = '';
-        catch(:no_such_key) { found = sub_lookup(key, lookup_invocation, segments, value) }
-        value = found;
+    if @scope_interpolations.empty?
+      true
+    else
+      scope = lookup_invocation.scope
+      lookup_invocation.without_explain do
+        @scope_interpolations.all? do |key, root_key, segments, old_value|
+          value = scope[root_key]
+          unless value.nil? || segments.empty?
+            found = '';
+            catch(:no_such_key) { found = sub_lookup(key, lookup_invocation, segments, value) }
+            value = found;
+          end
+          old_value.eql?(value)
+        end
       end
-      old_value.eql?(value)
     end
   end
 
@@ -297,9 +314,19 @@ class HieraConfigV3 < HieraConfig
     [@config[KEY_BACKENDS]].flatten.each do |backend|
       raise Puppet::DataBinding::LookupError, "#{@config_path}: Backend '#{backend}' defined more than once" if data_providers.include?(backend)
       original_paths = [@config[KEY_HIERARCHY]].flatten
-      backend_config = @config[backend] || EMPTY_HASH
-      datadir = Pathname(interpolate(backend_config[KEY_DATADIR] || default_datadir, lookup_invocation, false))
-      ext = backend == 'hocon' ? '.conf' : ".#{backend}"
+      backend_config = @config[backend]
+      if backend_config.nil?
+        backend_config = EMPTY_HASH
+      else
+        backend_config = interpolate(backend_config, lookup_invocation, false)
+      end
+      datadir = Pathname(backend_config[KEY_DATADIR] || interpolate(default_datadir, lookup_invocation, false))
+      ext = backend_config[KEY_EXTENSION]
+      if ext.nil?
+        ext = backend == 'hocon' ? '.conf' : ".#{backend}"
+      else
+        ext = ".#{ext}"
+      end
       paths = resolve_paths(datadir, original_paths, lookup_invocation, @config_path.nil?, ext)
       data_providers[backend] = case
       when backend == 'json', backend == 'yaml'
@@ -510,6 +537,7 @@ class HieraConfigV5 < HieraConfig
       end
 
       entry_datadir = @config_root + (he[KEY_DATADIR] || datadir)
+      entry_datadir = Pathname(interpolate(entry_datadir.to_s, lookup_invocation, false))
       location_key = LOCATION_KEYS.find { |key| he.include?(key) }
       locations = case location_key
       when KEY_PATHS
